# -*- coding: utf-8 -*-
"""
市场基类
"""
from abc import ABC, abstractmethod
from typing import Tuple, Optional
import pandas as pd
from ..cache import Cache
from ..data_sources.base import DataSource


class Market(ABC):
    """市场基类"""
    
    def __init__(self, data_source: DataSource, cache: Optional[Cache] = None):
        """
        初始化市场
        
        Args:
            data_source: 数据源
            cache: 缓存对象
        """
        self.data_source = data_source
        self.cache = cache
    
    @abstractmethod
    def get_price(self, symbol: str, sdate: str = '', edate: str = '', 
                  freq: str = 'day', days: int = 320, fq: str = 'qfq') -> Tuple[str, str, pd.DataFrame]:
        """获取价格数据"""
        pass
    
    @abstractmethod
    def normalize_symbol(self, symbol: str) -> str:
        """标准化股票代码"""
        pass
    
    def _get_cached(self, key: str) -> Optional[Tuple[str, str, pd.DataFrame]]:
        """从缓存获取数据"""
        if self.cache:
            cached = self.cache.get(key)
            if cached:
                return cached
        return None
    
    def _put_cache(self, key: str, value: Tuple[str, str, pd.DataFrame]) -> None:
        """存入缓存"""
        if self.cache:
            self.cache.put(key, value)




# -*- coding: utf-8 -*-
"""
缓存模块测试
"""
import unittest
import time
from rquote.cache import MemoryCache, DictCache, Cache


class TestMemoryCache(unittest.TestCase):
    """内存缓存测试"""
    
    def setUp(self):
        """设置测试环境"""
        self.cache = MemoryCache(ttl=1)  # 1秒过期
    
    def test_put_get(self):
        """测试存储和获取"""
        self.cache.put('key1', 'value1')
        self.assertEqual(self.cache.get('key1'), 'value1')
    
    def test_expire(self):
        """测试过期"""
        self.cache.put('key2', 'value2', ttl=0.1)  # 0.1秒过期
        time.sleep(0.2)
        self.assertIsNone(self.cache.get('key2'))
    
    def test_delete(self):
        """测试删除"""
        self.cache.put('key3', 'value3')
        self.cache.delete('key3')
        self.assertIsNone(self.cache.get('key3'))
    
    def test_clear(self):
        """测试清空"""
        self.cache.put('key4', 'value4')
        self.cache.put('key5', 'value5')
        self.cache.clear()
        self.assertEqual(self.cache.size(), 0)
    
    def test_size(self):
        """测试大小"""
        self.cache.put('key6', 'value6')
        self.cache.put('key7', 'value7')
        self.assertEqual(self.cache.size(), 2)


class TestDictCache(unittest.TestCase):
    """字典缓存测试"""
    
    def setUp(self):
        """设置测试环境"""
        self.dict_cache = {}
        self.cache = DictCache(self.dict_cache)
    
    def test_put_get(self):
        """测试存储和获取"""
        self.cache.put('key1', 'value1')
        self.assertEqual(self.cache.get('key1'), 'value1')
        self.assertEqual(self.dict_cache['key1'], 'value1')
    
    def test_delete(self):
        """测试删除"""
        self.cache.put('key2', 'value2')
        self.cache.delete('key2')
        self.assertIsNone(self.cache.get('key2'))


if __name__ == '__main__':
    unittest.main()




# -*- coding: utf-8 -*-
"""
港股市场
"""
import pandas as pd
from typing import Tuple
from .base import Market
from ..parsers import KlineParser
from ..exceptions import DataSourceError, ParseError
from ..utils import logger


class HKStockMarket(Market):
    """港股市场"""
    
    def normalize_symbol(self, symbol: str) -> str:
        """标准化港股代码"""
        if not symbol.startswith('hk'):
            return 'hk' + symbol
        return symbol
    
    def get_price(self, symbol: str, sdate: str = '', edate: str = '', 
                  freq: str = 'day', days: int = 320, fq: str = 'qfq') -> Tuple[str, str, pd.DataFrame]:
        """获取港股价格数据"""
        symbol = self.normalize_symbol(symbol)
        
        cache_key = f"{symbol}:{sdate}:{edate}:{freq}:{days}:{fq}"
        cached = self._get_cached(cache_key)
        if cached:
            return cached
        
        try:
            raw_data = self.data_source.fetch_kline(
                symbol, freq=freq, sdate=sdate, edate=edate, days=days, fq=fq
            )
            
            parser = KlineParser()
            name, df = parser.parse_tencent_kline(raw_data, symbol, fq=fq)
            
            result = (symbol, name, df)
            self._put_cache(cache_key, result)
            return result
        except (DataSourceError, ParseError) as e:
            logger.warning(f'Failed to fetch {symbol}: {e}')
            raise




# -*- coding: utf-8 -*-
"""
股票信息相关API
"""
import json
from typing import List
from ..utils import hget
from ..exceptions import HTTPError


def get_stock_concepts(i: str) -> List[str]:
    """
    获取指定股票所属的概念板块
    
    Args:
        i: 股票代码
    
    Returns:
        概念代码列表
    """
    i = {'6': 'sh', '0': 'sz', '3': 'sz'}.get(i[0], '') + i if i[0] in ['6', '0', '3'] else i
    url = f'https://proxy.finance.qq.com/ifzqgtimg/appstock/app/stockinfo/plateNew?code={i}&app=wzq&zdf=1'
    a = hget(url)
    if not a:
        raise HTTPError('Failed to fetch concepts from QQ Finance')
    data = json.loads(a.text)
    if data.get('code') != 0:
        raise HTTPError('API returned error: {}'.format(data.get('msg', 'Unknown error')))
    return data.get('data', {}).get('concept', [])


def get_stock_industry(i: str) -> List[str]:
    """
    获取指定股票所属的行业板块
    
    Args:
        i: 股票代码
    
    Returns:
        行业代码列表
    """
    i = {'6': 'sh', '0': 'sz', '3': 'sz'}.get(i[0], '') + i if i[0] in ['6', '0', '3'] else i
    url = f'https://proxy.finance.qq.com/ifzqgtimg/appstock/app/stockinfo/plateNew?code={i}&app=wzq&zdf=1'
    a = hget(url)
    if not a:
        raise HTTPError('Failed to fetch industry from QQ Finance')
    data = json.loads(a.text)
    if data.get('code') != 0:
        raise HTTPError('API returned error: {}'.format(data.get('msg', 'Unknown error')))
    return data.get('data', {}).get('plate', [])




# -*- coding: utf-8 -*-
"""
A股市场
"""
import json
import base64
import pandas as pd
from typing import Tuple
from .base import Market
from ..parsers import KlineParser
from ..exceptions import DataSourceError, ParseError
from ..utils import hget, logger


class CNStockMarket(Market):
    """A股市场"""
    
    def normalize_symbol(self, symbol: str) -> str:
        """标准化A股代码"""
        if symbol[0] in ['0', '1', '3', '5', '6']:
            prefix = 'sh' if symbol[0] in ['5', '6'] else 'sz'
            return prefix + symbol
        return symbol
    
    def get_price(self, symbol: str, sdate: str = '', edate: str = '', 
                  freq: str = 'day', days: int = 320, fq: str = 'qfq') -> Tuple[str, str, pd.DataFrame]:
        """获取A股价格数据"""
        symbol = self.normalize_symbol(symbol)
        
        # 检查缓存
        cache_key = f"{symbol}:{sdate}:{edate}:{freq}:{days}:{fq}"
        cached = self._get_cached(cache_key)
        if cached:
            return cached
        
        # 特殊处理BK（板块）代码
        if symbol[:2] == 'BK':
            return self._get_bk_price(symbol)
        
        # 特殊处理PT代码
        if symbol[:2] == 'pt':
            return self._get_pt_price(symbol, sdate, edate, freq, days, fq)
        
        # 使用数据源获取数据
        try:
            raw_data = self.data_source.fetch_kline(
                symbol, freq=freq, sdate=sdate, edate=edate, days=days, fq=fq
            )
            
            # 使用解析器解析
            parser = KlineParser()
            name, df = parser.parse_tencent_kline(raw_data, symbol)
            
            result = (symbol, name, df)
            self._put_cache(cache_key, result)
            return result
        except (DataSourceError, ParseError) as e:
            logger.warning(f'Failed to fetch {symbol} using new architecture: {e}')
            # 降级到旧方法
            return self._get_price_fallback(symbol, sdate, edate, freq, days, fq)
    
    def _get_bk_price(self, symbol: str) -> Tuple[str, str, pd.DataFrame]:
        """获取板块价格（BK开头）"""
        try:
            url = base64.b64decode('aHR0cDovL3B1c2gyaGlzLmVhc3' +
                                   'Rtb25leS5jb20vYXBpL3F0L3N0b2NrL2tsaW5lL2dldD9jYj1qUX' +
                                   'VlcnkxMTI0MDIyNTY2NDQ1ODczNzY2OTcyXzE2MTc4NjQ1NjgxMz' +
                                   'Emc2VjaWQ9OTAu').decode() + symbol + \
                              '&fields1=f1%2Cf2%2Cf3%2Cf4%2Cf5' + \
                              '&fields2=f51%2Cf52%2Cf53%2Cf54%2Cf55%2Cf56%2Cf57%2Cf58' + \
                              '&klt=101&fqt=0&beg=19900101&end=20990101&_=1'
            response = hget(url)
            if not response:
                logger.warning(f'{symbol} hget failed')
                return symbol, 'None', pd.DataFrame([])
            
            data = json.loads(response.text.split('jQuery1124022566445873766972_1617864568131(')[1][:-2])
            if not data.get('data'):
                logger.warning(f'{symbol} data empty')
                return symbol, 'None', pd.DataFrame([])
            
            name = data['data']['name']
            df = pd.DataFrame([i.split(',') for i in data['data']['klines']], 
                             columns=['date', 'open', 'close', 'high', 'low', 'vol', 'money', 'p'])
            df = df.set_index(['date'])
            # 转换数值列
            for col in ['open', 'close', 'high', 'low', 'vol', 'money', 'p']:
                df[col] = pd.to_numeric(df[col], errors='coerce')
            
            result = (symbol, name, df)
            self._put_cache(symbol, result)
            return result
        except Exception as e:
            logger.warning(f'error fetching {symbol}, err: {e}')
            return symbol, 'None', pd.DataFrame([])
    
    def _get_pt_price(self, symbol: str, sdate: str, edate: str, 
                     freq: str, days: int, fq: str) -> Tuple[str, str, pd.DataFrame]:
        """获取PT代码价格"""
        try:
            url = f'https://proxy.finance.qq.com/ifzqgtimg/appstock/app/newfqkline/get?_var=kline_dayqfq&param={symbol},{freq},{sdate},{edate},{days},{fq}'
            response = hget(url)
            if not response:
                logger.warning(f'{symbol} hget failed')
                return symbol, 'None', pd.DataFrame([])
            
            response_text = response.text
            json_start = response_text.find('{')
            if json_start == -1:
                logger.warning(f'{symbol} invalid response format')
                return symbol, 'None', pd.DataFrame([])
            
            data = json.loads(response_text[json_start:])
            if data.get('code') != 0:
                logger.warning(f'{symbol} API returned error: {data.get("msg", "Unknown error")}')
                return symbol, 'None', pd.DataFrame([])
            
            # 使用解析器
            try:
                parser = KlineParser()
                name, df = parser.parse_tencent_kline(data, symbol)
                result = (symbol, name, df)
                self._put_cache(f"{symbol}:{sdate}:{edate}:{freq}", result)
                return result
            except Exception as e:
                logger.warning(f'Failed to parse {symbol}, using fallback: {e}')
                # 降级处理
                symbol_data = data.get('data', {}).get(symbol, {})
                if not symbol_data:
                    return symbol, 'None', pd.DataFrame([])
                
                tk = None
                for tkt in ['day', 'qfqday', 'hfqday', 'week', 'qfqweek', 'hfqweek',
                            'month', 'qfqmonth', 'hfqmonth']:
                    if tkt in symbol_data:
                        tk = tkt
                        break
                
                if not tk:
                    return symbol, 'None', pd.DataFrame([])
                
                name = ''
                if 'qt' in symbol_data and symbol in symbol_data['qt']:
                    name = symbol_data['qt'][symbol][1] if len(symbol_data['qt'][symbol]) > 1 else ''
                
                kline_data = symbol_data[tk]
                df = pd.DataFrame([j[:6] for j in kline_data],
                                 columns=['date', 'open', 'close', 'high', 'low', 'vol']).set_index('date')
                for col in ['open', 'high', 'low', 'close', 'vol']:
                    df[col] = pd.to_numeric(df[col], errors='coerce')
                
                result = (symbol, name, df)
                self._put_cache(f"{symbol}:{sdate}:{edate}:{freq}", result)
                return result
        except Exception as e:
            logger.warning(f'error fetching {symbol}, err: {e}')
            return symbol, 'None', pd.DataFrame([])
    
    def _get_price_fallback(self, symbol: str, sdate: str, edate: str, 
                           freq: str, days: int, fq: str) -> Tuple[str, str, pd.DataFrame]:
        """降级方法（旧实现）"""
        from ..utils import hget
        import json
        
        url = f'https://web.ifzq.gtimg.cn/appstock/app/newfqkline/get?param={symbol},{freq},{sdate},{edate},{days},{fq}'
        response = hget(url)
        if not response:
            raise DataSourceError(f'Failed to fetch data for {symbol}')
        
        data = json.loads(response.text)['data'][symbol]
        name = ''
        for tkt in ['day', 'qfqday', 'hfqday', 'week', 'qfqweek', 'hfqweek',
                    'month', 'qfqmonth', 'hfqmonth']:
            if tkt in data:
                tk = tkt
                break
        
        df = pd.DataFrame([j[:6] for j in data[tk]],
                         columns=['date', 'open', 'close', 'high', 'low', 'vol']).set_index('date')
        for col in ['open', 'high', 'low', 'close', 'vol']:
            df[col] = pd.to_numeric(df[col], errors='coerce')
        if 'qt' in data:
            name = data['qt'][symbol][1]
        
        result = (symbol, name, df)
        cache_key = f"{symbol}:{sdate}:{edate}:{freq}:{days}:{fq}"
        self._put_cache(cache_key, result)
        return result




# -*- coding: utf-8 -*-
"""
日志工具
"""
import logging


def setup_logger():
    """设置日志记录器"""
    logger = logging.getLogger('rquote')
    if not logger.handlers:
        logger.setLevel(logging.INFO)
        file_handler = logging.FileHandler('/tmp/rquote.log')
        
        formatter = logging.Formatter('%(asctime)-15s:%(lineno)s %(message)s')
        file_handler.setFormatter(formatter)
        
        logger.addHandler(file_handler)
        logger.addHandler(logging.StreamHandler())
    
    return logger


logger = setup_logger()




# -*- coding: utf-8 -*-
"""
列表相关API
"""
import json
import re
import base64
import time
from ..utils import hget, logger
from ..exceptions import HTTPError


def get_cn_stock_list(money_min=2e8):
    """
    获取A股股票列表
    
    Args:
        money_min: 最小成交额
    
    Returns:
        股票列表
    """
    offset = 0
    count = 200  # max, or error
    df = []
    while not df or float(df[-1]['turnover'])*1e4 > money_min:
        a = hget(
            'https://proxy.finance.qq.com/cgi/cgi-bin/rank/hs/getBoardRankList?_appver=11.17.0'+
            f'&board_code=aStock&sort_type=turnover&direct=down&offset={offset}&count={count}'
        )
        if a:
            a = json.loads(a.text)
            if a['data']['rank_list']:
                df.extend(a['data']['rank_list'])
                offset += count
            else:
                break
    return df


def get_hk_stocks_500():
    """
    获取港股前500只股票列表
    
    Returns:
        股票列表
    """
    a = hget(
        'https://stock.gtimg.cn/data/hk_rank.php?board=main_all&metric=amount&' +
        'pageSize=500&reqPage=1&order=desc&var_name=list_data')
    if a:
        a = [i.split('~') for i in json.loads(a.text.split('list_data=')[1])['data']['page_data']]
    return a


def get_us_stocks(k=100):
    """
    获取美股列表
    
    Args:
        k: 获取数量
    
    Returns:
        股票列表
    """
    uscands = []
    page_n = k//20 + 1
    for page in range(1, page_n+1):
        a = hget(
            "https://stock.finance.sina.com.cn/usstock/api/jsonp.php/IO.XSRV2."+
            f"CallbackList['f0j3ltzVzdo2Fo4p']/US_CategoryService.getList?page={page}"+
            "&num=20&sort=&asc=0&market=&id=")
        if a:
            uslist = json.loads(a.text.split('(',1)[1][:-2])['data']
            uscands.extend(uslist)
    return uscands


def get_cn_fund_list():
    """
    获取A股ETF基金列表
    
    Returns:
        基金列表，格式: [code, name, change, amount, price]
    """
    a = hget(base64.b64decode('aHR0cDovL3ZpcC5zdG9jay5maW5hbmNlLnNpbmEuY29tL'+
        'mNuL3F1b3Rlc19zZXJ2aWNlL2FwaS9qc29ucC5waHAvSU8uWFNSVjIuQ2FsbGJhY2tMaX'+
        'N0WydrMldhekswNk5Rd2xoeVh2J10vTWFya2V0X0NlbnRlci5nZXRIUU5vZGVEYXRhU2l'+
        'tcGxlP3BhZ2U9MSZudW09MTAwMCZzb3J0PWFtb3VudCZhc2M9MCZub2RlPWV0Zl9ocV9m'+
        'dW5kJiU1Qm9iamVjdCUyMEhUTUxEaXZFbGVtZW50JTVEPXhtNGkw').decode())
    if a:
        fundcands = [[i['symbol'], i['name'], i['changepercent'], i['amount'], i['trade']]
                     for i in json.loads(a.text.split('k2WazK06NQwlhyXv')[1][3:-2])]
    return fundcands


def get_cn_future_list():
    """
    获取国内期货合约列表
    
    Returns:
        期货代码列表，带fu前缀
    """
    a = hget('https://finance.sina.com.cn/futuremarket/')
    if a:
        futurelist_active = [
            'fu' + i for i in re.findall(r'quotes/(.*?\d+).shtml', a.text)]
    return futurelist_active


def get_all_industries():
    """
    获取所有行业板块列表
    
    Returns:
        行业列表，格式: [code, name, change, amount, price, sina_sw2_id]
    """
    try:
        url = 'https://proxy.finance.qq.com/cgi/cgi-bin/rank/pt/getRank?board_type=hy2&sort_type=price&direct=down&offset=0&count=200'
        a = hget(url)
        if not a:
            raise HTTPError('Failed to fetch industries from QQ Finance')
        data = json.loads(a.text)
        if data.get('code') != 0:
            logger.warning('API returned error: {}'.format(data.get('msg', 'Unknown error')))
            return []
        
        rank_list = data.get('data', {}).get('rank_list', [])
        industries = [
            [item['code'], item['name'], item.get('zdf', '0'), 
             item.get('zllr', '0'), item.get('zxj', '0')]
            for item in rank_list
        ]
        logger.debug('get industries {}'.format(len(industries)))
    except Exception as e:
        logger.warning('Error parsing industries data: {}'.format(e))
        industries = []
    
    try:
        url = 'https://vip.stock.finance.sina.com.cn/quotes_service/api/json_v2.php/Market_Center.getHQNodes'
        sina_industries = hget(url)
        if not sina_industries:
            raise HTTPError('Failed to fetch industries from Sina Finance')
        data = json.loads(sina_industries.text)
        sina_industries = data[1][0][1]
        sina_sw2 = sina_industries[3][1]
        sina_sw2_dict = {i[0]:i[2] for i in sina_sw2}
    except Exception as e:
        logger.warning(f'Error parsing sina industries data: {e}')
        sina_sw2_dict = {}
    
    # 利用sina_sw2_dict给industries添加sina_sw2_id
    for industry in industries:
        industry.append(sina_sw2_dict.get(industry[1], ''))
    return industries


def get_industry_stocks(node):
    """
    获取指定行业板块的股票列表
    
    Args:
        node: 行业节点名称
    
    Returns:
        股票列表
    """
    url = f'https://vip.stock.finance.sina.com.cn/quotes_service/api/json_v2.php/'+\
    f'Market_Center.getHQNodeData?page=1&num=40&sort=symbol&asc=1&node={node}&symbol=&_s_r_a=init'
    a = hget(url)
    if not a:
        raise HTTPError('Failed to fetch industry stocks from Sina Finance')
    data = json.loads(a.text)
    return data

